@@ -4,6 +4,7 @@ from dexy.utils import file_exists
 from dexy.utils import defaults
 import dexy.wrapper
 import os
+import yaml
 
 RENAME_PARAMS = {
         'artifactsdir' : 'artifacts_dir',
@@ -84,8 +85,8 @@ def config_args(modargs):
                 conf_args = parse_json(f.read())
             else:
                 raise dexy.exceptions.UserFeedback("Don't know how to load config from '%s'" % config_file)
-
-            kwargs.update(conf_args)
+            if conf_args:
+                kwargs.update(conf_args)
 
     if cliargs: # cliargs may be False
         for k in cliargs.keys(): kwargs[k] = modargs[k]
@@ -94,20 +95,69 @@ def config_args(modargs):
 
     return kwargs
 
+def import_plugins_from_local_yaml_file(import_target):
+    if os.path.exists(import_target):
+        with open(import_target, 'rb') as f:
+            yaml_content = yaml.safe_load(f.read())
+
+        for alias, info_dict in yaml_content.iteritems():
+            if ":" in alias:
+                prefix, alias = alias.split(":")
+            else:
+                prefix = 'filter'
+
+            plugin_classes = {
+                'filter' : dexy.filter.Filter,
+                'reporter' : dexy.reporter.Reporter
+            }
+
+            if not prefix in plugin_classes:
+                msg = "'%s' not found, available aliases are %s"
+                args = (prefix, ", ".join(plugin_classes.keys()))
+                raise dexy.exceptions.UserFeedback(msg % args)
+
+            cls = plugin_classes[prefix]
+
+            if alias in cls.plugins:
+                existing_plugin = cls.plugins[alias]
+                plugin_settings = existing_plugin[1]
+                plugin_settings.update(info_dict)
+                cls.plugins[alias] = (existing_plugin[0], plugin_settings)
+            else:
+                cls.register_plugins_from_yaml_content({alias : info_dict})
+
+    else:
+        # Don't raise exception if default files don't exist.
+        if not import_target in ('dexyplugin.yaml', 'dexyplugins.yaml',):
+            msg = "Could not find YAML file named '%s'" % import_target
+            raise dexy.exceptions.UserFeedback(msg)
+
+def import_plugins_from_local_python_file(import_target):
+    if os.path.exists(import_target):
+        import imp
+        imp.load_source("custom_plugins", import_target)
+    else:
+        # Don't raise exception if default files don't exist.
+        if not import_target in ('dexyplugin.py', 'dexyplugins.py',):
+            msg = "Could not find python file named '%s'" % import_target
+            raise dexy.exceptions.UserFeedback(msg)
+
+def import_plugins_from_python_package(import_target):
+    try:
+        __import__(import_target)
+    except ImportError:
+        msg = "Could not find installed python package named '%s'" % import_target
+        raise dexy.exceptions.UserFeedback(msg)
+
 def import_extra_plugins(kwargs):
     if kwargs.get('plugins'):
         for import_target in kwargs.get('plugins').split():
-            print "loading plugin", import_target
-            # TODO look for files
-            try:
-                __import__(import_target)
-            except ImportError as e:
-                # try looking for file
-                if os.path.exists(import_target):
-                    import imp
-                    imp.load_source("custom_plugins", import_target)
-                else:
-                    raise e
+            if import_target.endswith('.yaml'):
+                import_plugins_from_local_yaml_file(import_target)
+            elif import_target.endswith('.py'):
+                import_plugins_from_local_python_file(import_target)
+            else:
+                import_plugins_from_python_package(import_target)
 
 def init_wrapper(modargs):
     kwargs = config_args(modargs)
@@ -119,17 +169,15 @@ def init_wrapper(modargs):
 def template_text(
         alias=None
     ):
-    template = dexy.template.Template.aliases[alias]
-    for batch in template.dexy():
+    template = dexy.template.Template.create_instance(alias)
+    for wrapper in template.dexy(True):
         man_doc_key = 'doc:dexy.rst|jinja|rst2man'
-        if man_doc_key in batch.lookup_table:
-            man_doc = batch.nodes[man_doc_key].output_data().storage.data_file()
+        if man_doc_key in wrapper.nodes:
+            man_doc = wrapper.nodes[man_doc_key].output_data().storage.data_file()
 
-            command = "man %s" % man_doc
             import subprocess
             proc = subprocess.Popen(
-                       command,
-                       shell=True,
+                       ["man", man_doc],
                        stdout=subprocess.PIPE,
                        stderr=subprocess.STDOUT
                    )
