@@ -22,27 +22,78 @@ class Filter(dexy.plugin.Plugin):
             'help', 'nodoc'
             ]
     _settings = {
-            'add-new-files' : ("""Whether to add new files that have been
-                created as side effects of running this filter.""", False),
-            'exclude-add-new-files' : ("""Patterns to exclude when adding new
-                side effect files.""" , ''),
-            'additional-doc-filters' : ('', {}),
-            'examples' : ("Templates which should be used as examples for this filter.", []),
-            'ext' : ('Extension to output.', None),
-            'help' : ('Help string for filter, if not already specified as a class docstring.', None),
-            'include-in-workspaces' : ("Allow overriding whether a document should be used when populating workspaces for other documents.", False),
-            'input-extensions' : ("List of extensions which this filter can accept as input.", [".*"]),
-            'keep-originals' : ('', False),
-            'nodoc' : ("Whether filter should be excluded from documentation.", False),
-            'output' : ("Whether to output results of this filter by default by reporters such as 'output' or 'website'.", False),
-            'output-data-type' : ("Alias of data type to use to store filter output.", "generic"),
-            'output-extensions' : ("List of extensions which this filter can produce as output.", [".*"]),
-            'preserve-prior-data-class' : ('', False),
-            'require-output' : ("Should dexy raise an exception if no output is produced by this filter?", True),
-            'tags' : ("Tags which describe the filter.", []),
-            'variables' : ('', {}),
-            'vars' : ('', {}),
-            'workspace-exclude-filters' : ("Filters whose output should be excluded from workspace.", ['pyg'])
+            'added-in-version' : (
+                "Dexy version when this filter was first available.",
+                ''),
+            'add-new-files' : (
+                "Whether to add new files that have been created as side effects of running this filter.",
+                False),
+            'exclude-add-new-files' : (
+                "Patterns to exclude when adding new side effect files." ,
+                ''),
+            'additional-doc-filters' : (
+                "Filters to apply to additional documents created as side effects.",
+                {}),
+            'additional-doc-settings' : (
+                "Settings to apply to additional documents created as side effects.",
+                {}),
+            'examples' : (
+                "Templates which should be used as examples for this filter.",
+                []),
+            'ext' : (
+                'File extension to output.',
+                None),
+            'extension-map' : (
+                "Dictionary mapping input extensions to default output extensions.",
+                None),
+            'help' : (
+                'Help string for filter, if not already specified as a class docstring.',
+                None),
+            'include-in-workspaces' : (
+                "Allow overriding whether a document should be used when populating workspaces for other documents.",
+                False),
+            'input-extensions' : (
+                "List of extensions which this filter can accept as input.",
+                [".*"]),
+            'keep-originals' : (
+                "Whether, if additional-doc-filters are specified, the original unmodified docs should also be added.",
+                False),
+            'mkdir' : (
+                "A directory which should be created in working dir.",
+                None),
+            'mkdirs' : (
+                "A list of directories which should be created in working dir.",
+                []),
+            'nodoc' : (
+                "Whether filter should be excluded from documentation.",
+                False),
+            'output' : (
+                "Whether to output results of this filter by default by reporters such as 'output' or 'website'.",
+                False),
+            'output-data-type' : (
+                "Alias of data type to use to store filter output.",
+                "generic"),
+            'output-extensions' : (
+                "List of extensions which this filter can produce as output.",
+                [".*"]),
+            'preserve-prior-data-class' : (
+                "Whether output data class should be set to match the input data class.",
+                False),
+            'require-output' : (
+                "Should dexy raise an exception if no output is produced by this filter?",
+                True),
+            'tags' : (
+                "Tags which describe the filter.",
+                []),
+            'variables' : (
+                'A dictionary of variable names and values to make available to this filter.',
+                {}),
+            'vars' : (
+                'A dictionary of variable names and values to make available to this filter.',
+                {}),
+            'workspace-exclude-filters' : (
+                "Filters whose output should be excluded from workspace.",
+                ['pyg'])
             }
 
     def __init__(self, doc=None):
@@ -144,25 +195,40 @@ class Filter(dexy.plugin.Plugin):
         else:
             # User has not specified desired extension, and we don't output wildcards,
             # figure out extension based on next filter in sequence, if any.
+            ext_from_map = None
+            if self.setting('extension-map'):
+                ext_from_map = self.setting('extension-map')[prev_ext]
+
+            next_filter_accepts = [".*"]
             if self.next_filter:
                 next_filter_accepts = self.next_filter.setting('input-extensions')
 
-                if ".*" in next_filter_accepts:
-                    self.ext = i_output[0]
+            if ".*" in next_filter_accepts:
+                if ext_from_map:
+                    self.ext = ext_from_map
                 else:
-                    if set(i_output).isdisjoint(set(next_filter_accepts)):
-                        msg = "Filter %s can't go after filter %s, no file extensions in common."
-                        raise dexy.exceptions.UserFeedback(msg % (self.next_filter.alias, self.alias))
+                    self.ext = i_output[0]
 
-                    for e in i_output:
-                        if e in next_filter_accepts:
-                            self.ext = e
+            elif ext_from_map:
+                if not ext_from_map in next_filter_accepts:
+                    msg = "Filter %s wants to output %s but %s doesn't accept this format."
+                    msgargs = (self.alias, ext_from_map, self.next_filter.alias)
+                    raise dexy.exceptions.UserFeedback(msg % msgargs)
+
+                self.ext = ext_from_map
 
-                    if not self.ext:
-                        msg = "no file extension found but checked already for disjointed, should not be here"
-                        raise dexy.exceptions.InternalDexyProblem(msg)
             else:
-                self.ext = i_output[0]
+                if set(i_output).isdisjoint(set(next_filter_accepts)):
+                    msg = "Filter %s can't go after filter %s, no file extensions in common."
+                    raise dexy.exceptions.UserFeedback(msg % (self.next_filter.alias, self.alias))
+
+                for e in i_output:
+                    if e in next_filter_accepts:
+                        self.ext = e
+
+                if not self.ext:
+                    msg = "no file extension found but checked already for disjointed, should not be here"
+                    raise dexy.exceptions.InternalDexyProblem(msg)
 
     def templates(self):
         """
@@ -172,11 +238,6 @@ class Filter(dexy.plugin.Plugin):
         templates = [dexy.template.Template.create_instance(a) for a in self.setting('examples')]
         return templates
 
-    def filter_specific_settings(self):
-        nodoc = self.nodoc_settings
-        base = dexy.filter.Filter._settings
-        return dict((k, v) for k, v in self._settings.iteritems() if not k in nodoc and not k in base)
-
     def key_with_class(self):
         return "%s:%s" % (self.__class__.__name__, self.key)
 
@@ -213,10 +274,31 @@ class Filter(dexy.plugin.Plugin):
         if not posixpath.sep in doc_name:
             doc_name = posixpath.join(self.input_data.parent_dir(), doc_name)
 
+        doc_ext = os.path.splitext(doc_name)[1]
+
         additional_doc_filters = self.setting('additional-doc-filters')
         self.log_debug("additional-doc-filters are %s" % additional_doc_filters)
 
-        doc_ext = os.path.splitext(doc_name)[1]
+        
+        additional_doc_settings = self.setting('additional-doc-settings')
+
+        settings = None
+        if isinstance(additional_doc_settings, list):
+            # figure out which settings to apply based on file extension
+            for pattern, settings in additional_doc_settings:
+                if doc_ext == pattern or pattern == ".*":
+                    break
+        elif isinstance(additional_doc_settings, dict):
+            settings = additional_doc_settings
+
+        else:
+            raise Exception("Unexpected type %s" % type(settings))
+
+        if doc_args:
+            settings.update(doc_args)
+
+        self.log_debug("additional-doc-settings are %s" % settings)
+
 
         def create_doc(name, filters, contents, args=None):
             if filters:
@@ -234,28 +316,36 @@ class Filter(dexy.plugin.Plugin):
                     contents=contents,
                     **args
                     )
+
+            doc.output_data().setup()
+            doc.output_data().storage.connect()
+
             self.doc.add_additional_doc(doc)
             return doc
 
-        if isinstance(additional_doc_filters, dict):
-            filters = additional_doc_filters.get(doc_ext, '')
-        elif isinstance(additional_doc_filters, basestring):
+        if isinstance(additional_doc_filters, basestring):
             filters = additional_doc_filters
         elif isinstance(additional_doc_filters, list):
-            for filters in additional_doc_filters:
-                doc = create_doc(doc_name, filters, doc_contents)
-            # make filters empty so we don't make more docs later
-            filters = []
+            for f in additional_doc_filters:
+                create_doc(doc_name, f, doc_contents, settings)
+            filters = ''
+        elif isinstance(additional_doc_filters, dict):
+            filters = additional_doc_filters.get(doc_ext, '')
+            if isinstance(filters, list):
+                for f in filters:
+                    create_doc(doc_name, f, doc_contents, settings)
+                filters = ''
         else:
             msg = "received unexpected additional_doc_filters arg class %s"
             msgargs = additional_doc_filters.__class__.__name__
             raise dexy.exceptions.InternalDexyProblem(msg % msgargs)
 
+
         if len(filters) == 0 or self.setting('keep-originals'):
-            doc = create_doc(doc_name, '', doc_contents, doc_args)
+            doc = create_doc(doc_name, '', doc_contents, settings)
 
         if len(filters) > 0:
-            doc = create_doc(doc_name, filters, doc_contents, doc_args)
+            doc = create_doc(doc_name, filters, doc_contents, settings)
 
         return doc
 
@@ -321,6 +411,22 @@ class Filter(dexy.plugin.Plugin):
         # include anything left over
         return True
 
+    def makedirs(self):
+        mkdirs = self.setting('mkdirs')
+
+        # mkdir should be a string, but handle either string or list
+        mkdir = self.setting('mkdir')
+        if mkdir:
+            if isinstance(mkdir, basestring):
+                mkdirs.append(mkdir)
+            else:
+                mkdirs.extend(mkdir)
+
+        for d in mkdirs:
+            dirpath = os.path.join(self.workspace(), d)
+            self.log_debug("Creating directory %s" % dirpath)
+            os.makedirs(dirpath)
+
     def populate_workspace(self):
         """
         Populates the workspace directory with inputs to the filter, under
@@ -341,6 +447,8 @@ class Filter(dexy.plugin.Plugin):
             msgargs = (os.path.abspath(wd), self.key,)
             raise dexy.exceptions.InternalDexyProblem(msg % msgargs)
 
+        self.makedirs()
+
         for inpt in self.doc.walk_input_docs():
             if not self.include_input_in_workspace(inpt):
                 self.log_debug("not populating workspace with input '%s'" % inpt.key)
@@ -413,9 +521,6 @@ class DexyFilter(Filter):
     """
     aliases = ['dexy']
 
-    def data_class_alias(self, ext):
-        return self.setting('output-data-type')
-
     def process(self):
         if hasattr(self, "process_text"):
             output = self.process_text(unicode(self.input_data))
